using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Diagnostics;

namespace d3d
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
 
        }

        // The Direct3D device.
        private Device device;

        #region "D3D Setup Code"

        // Data variables.
        private const int NUM_TRIANGLES = 12;
        private const int NUM_POINTS = 3 * NUM_TRIANGLES;

        // The vertex buffer that holds drawing data.
        private VertexBuffer m_VertexBuffer = null;

        // Initialize the graphics device. Return True if successful.
        public bool InitializeGraphics()
        {
            #region present parameters section   used to make device


            PresentParameters d3dpp = new PresentParameters();
            d3dpp.Windowed = true;
            d3dpp.SwapEffect = SwapEffect.Discard;
            //d3dpp.AutoDepthStencilFormat = DepthFormat.D16;
            //d3dpp.EnableAutoDepthStencil = true;

            #endregion   
            #region Make new device

            

            // Best: Hardware device and hardware vertex processing.
            try
            {
                device = new Device(0, DeviceType.Hardware, pic3d,
                    CreateFlags.HardwareVertexProcessing, d3dpp);
                Debug.WriteLine("Hardware, HardwareVertexProcessing");
            }
            catch { }

            // Good: Hardware device and software vertex processing.
            if (device == null)
            {
                try
                {
                    device = new Device(0, DeviceType.Hardware, pic3d,
                        CreateFlags.SoftwareVertexProcessing, d3dpp);
                    Debug.WriteLine("Hardware, SoftwareVertexProcessing");
                }
                catch { }
            }

            // Adequate?: Software device and software vertex processing.
            if (device == null)
            {
                try
                {
                    device = new Device(0, DeviceType.Reference, pic3d,
                        CreateFlags.SoftwareVertexProcessing, d3dpp);
                    Debug.WriteLine("Reference, SoftwareVertexProcessing");
                }
                catch (Exception ex)
                {
                    // If we still can't make a device, give up.
                    MessageBox.Show("Error initializing Direct3D\n\n" + ex.Message,
                        "Direct3D Error", MessageBoxButtons.OK);
                    return false;
                }
            }
            #endregion

            #region device.methods

            
            // Turn off D3D lighting because 
            // we set the vertex colors explicitly.
            device.RenderState.Lighting = false;

            // Cull triangles that are oriented counter clockwise.
            device.RenderState.CullMode = Cull.CounterClockwise;

            // Make points bigger so they're easy to see.
            device.RenderState.PointSize = 4;
            #endregion
            // Create the vertex data.
            CreateVertexBuffer();

            // We succeeded.
            return true;
        }

        // Create a vertex buffer for the device.
        public void CreateVertexBuffer()
        {
            // Create a buffer.
            m_VertexBuffer = new VertexBuffer(
                typeof(CustomVertex.PositionColored),
                NUM_POINTS+100, device, 0,
                CustomVertex.PositionColored.Format,
                Pool.Default);

            // Lock the vertex buffer. 
            // Lock returns an array of positionColored objects.
            CustomVertex.PositionColored[] vertices =
                (CustomVertex.PositionColored[])m_VertexBuffer.Lock(0, 0);

            // Make the vertexes.
            int i = 0;
            // Top +Y.
            MakeTriangle(vertices, ref i, Color.Green, 1, 1, -1,  -1, 1, -1,  -1, 1, 1);
            MakeTriangle(vertices, ref i, Color.Green, -1, 1, 1,   1, 1, 1,    1, 1, -1);

            // Bottom. -Y
            MakeTriangle(vertices, ref i, Color.White, 1, -1, -1, 1, -1, 1, -1, -1, 1);
            MakeTriangle(vertices, ref i, Color.White, -1, -1, 1, -1, -1, -1, 1, -1, -1);

            // Right +X.
            MakeTriangle(vertices, ref i, Color.Blue, 1, 1, -1, 1, 1, 1, 1, -1, 1);
            MakeTriangle(vertices, ref i, Color.Blue, 1, -1, 1, 1, -1, -1, 1, 1, -1);

            // Left -X.
            MakeTriangle(vertices, ref i, Color.Orange, -1, -1, -1, -1, -1, 1, -1, 1, 1);
            MakeTriangle(vertices, ref i, Color.Orange, -1, 1, 1, -1, 1, -1, -1, -1, -1);

            // Front -Z.
            MakeTriangle(vertices, ref i, Color.Red, -1, 1, -1,  1, 1, -1,  1, -1, -1);
            MakeTriangle(vertices, ref i, Color.Red, 1, -1, -1,  -1, -1, -1,  -1, 1, -1);

            // Back +Z.
            MakeTriangle(vertices, ref i, Color.Yellow, 1, 1, 1, -1, 1, 1, -1, -1, 1);
            MakeTriangle(vertices, ref i, Color.Yellow, -1, -1, 1, 1, -1, 1, 1, 1, 1);

            MakeTriangle(vertices, ref i, Color.Red, 2.5F, 1, -1, 4.5F, 1, -1,  4.5F, -1, -1);
            MakeTriangle(vertices, ref i, Color.Red, 4.5F, -1, -1,  2.5F, -1, -1,  2.5F, 1, -1);


            m_VertexBuffer.Unlock();
        }

        // Add a triangle to the vertex buffer.
        private void MakeTriangle(CustomVertex.PositionColored[] vertices, ref int i, Color clr,
            float x0, float y0, float z0,
            float x1, float y1, float z1,
            float x2, float y2, float z2)
        {
            vertices[i].X = x0;
            vertices[i].Y = y0;
            vertices[i].Z = z0;
            vertices[i].Color = clr.ToArgb();
            i++;

            vertices[i].X = x1;
            vertices[i].Y = y1;
            vertices[i].Z = z1;
            vertices[i].Color = clr.ToArgb();
            i++;

            vertices[i].X = x2;
            vertices[i].Y = y2;
            vertices[i].Z = z2;
            vertices[i].Color = clr.ToArgb();
            i++;
        }

        #endregion // D3D Setup Code

        #region "D3D Drawing Code"
        // Draw.
        public void Render()
        {
            // Clear the back buffer.
            device.Clear(ClearFlags.Target, Color.MidnightBlue, 1, 0);

            // Make a scene.
            device.BeginScene();

            // Draw stuff here...
            // Setup the world, view, and projection matrices.
            SetupMatrices();

            // Set the device//s data stream source (the vertex buffer).
            device.SetStreamSource(0, m_VertexBuffer, 0);

            // Tell the device the format of the vertices.
            device.VertexFormat = CustomVertex.PositionColored.Format;

            // Draw the primitives in the data stream.
            device.DrawPrimitives(PrimitiveType.TriangleList, 0, NUM_TRIANGLES+2);

            // End the scene and display.
            device.EndScene();
            device.Present();
        }
        float zangle = 0;
        // Setup the world, view, and projection matrices.
        private void SetupMatrices()
        {
            // World Matrix:
            // Rotate the object around the Y axis by
            // 2 * Pi radians per 4000 ticks (4 seconds).
            const int TICKS_PER_REV = 4000;
            double angle = Environment.TickCount * (2 * Math.PI) / TICKS_PER_REV;
            zangle = zangle + .001F;
            device.Transform.World = Matrix.RotationY((float)zangle);

            // View Matrix:
            // This is defined by giving:
            //       An eye point            (0, 3, -4)
            //       A point to look at      (0, 0, 0)
            //       An "up" direction       <0, 1, 0>
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 4, -8),
                new Vector3(0, 0, 0),
                new Vector3(0, 1, 0));

            // Projection Matrix:
            // Perspective transformation defined by:
            //       Field of view           Pi / 4
            //       Aspect ratio            1
            //       Near clipping plane     Z = 1
            //       Far clipping plane      Z = 100
            device.Transform.Projection =
                Matrix.PerspectiveFovLH((float)(Math.PI / 4), 1, 1, 100);
        }
        #endregion // D3D Drawing Code

       
    }
}